package org.beliaj.knots.io.impl;

import java.awt.Dimension;
import java.io.*;
import java.util.List;

import javax.xml.parsers.*;
import org.beliaj.knots.diagram.IKnotDiagram;
import org.beliaj.knots.diagram.ILine;
import org.beliaj.knots.diagram.IPoint;
import org.beliaj.knots.diagram.impl.KnotDiagram;
import org.beliaj.knots.io.IDiagramReader;

import org.xml.sax.*;
import org.xml.sax.helpers.*;

public class BasicDiagramReader implements IDiagramReader {
	
	private static class TheContentHandler extends DefaultHandler{
		
		IKnotDiagram myDiagram;
		
		private int[] myPointId;
		private int[] myLineId;
		int numb = 0;
		int numb2 = 0;

		private Dimension myDimension;
		
		TheContentHandler(IKnotDiagram d){
			super();
			this.myDiagram = d;
		}
		
		private Dimension getDimension(){
			return this.myDimension;
		}

		public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
			//System.out.println(qName);
			if (qName.equals("point")){
				//System.out.println(myPointId.size());
				this.myPointId[Integer.parseInt(atts.getValue("id"))] = this.numb;
				this.numb++;
				this.myDiagram.addPoint(Integer.parseInt(atts.getValue("x")), Integer.parseInt(atts.getValue("y")));
				//System.out.println(this.myDiagram);
			} else {
				if (qName.equals("line")){
					this.myLineId[Integer.parseInt(atts.getValue("id"))] = this.numb2;
					this.numb2++;
					List<IPoint> list = this.myDiagram.getPoints();
					this.myDiagram.connectPoints(list.get(
									this.myPointId[Integer.parseInt(atts.getValue("fpoint"))]), 
							list.get(
									this.myPointId[Integer.parseInt(atts.getValue("spoint"))]));
					//System.out.println(this.myDiagram);
				} else if (qName.equals("intersection")){
					ILine line1 = this.myDiagram.getLines().get(
							this.myLineId[Integer.parseInt(atts.getValue("upline"))]);
					ILine line2 = this.myDiagram.getLines().get(
							this.myLineId[Integer.parseInt(atts.getValue("downline"))]);
					this.myDiagram.setIntersectionType(line1, line2);
				} else if (qName.equals("size")){
					this.myDimension = new Dimension(Integer.parseInt(atts.getValue("width")), Integer.parseInt(atts.getValue("height")));
					//System.out.println(this.myDimension);
				} else if (qName.equals("points")){
					myPointId = new int[Integer.parseInt(atts.getValue("count"))];
					//System.out.println(Integer.parseInt(atts.getValue("count")) + " " + myPointId.size());
				} else if (qName.equals("lines")){
					myLineId = new int[Integer.parseInt(atts.getValue("count"))];
				}
			}
		}
		
	}
	
	private String myDiagramFile;
	
	private final static String SHEMA = "basicShema.xsd";

	private static final String DESCRIPTION = "simple knot diagram file type (*.KND)";

	private static final Object FILTER = "knd";
	
	XMLReader myReader;

	private Dimension myDimension;
	
	/**
	 * ,   XML
	 * @param fileName  XML 
	 * @throws SAXException 
	 * @throws ParserConfigurationException 
	 */
	public BasicDiagramReader(String fileName) throws SAXException, ParserConfigurationException{
		this.myDiagramFile = fileName;
		
		SAXParserFactory factory = SAXParserFactory.newInstance();
		
		//SchemaFactory sfactory = SchemaFactory.newInstance("http://www.w3.org/2001/XMLSchema");
		//Schema schema = null;
		//schema = sfactory.newSchema(new File(SHEMA));
		//factory.setSchema(schema);
		//factory.setValidating(true);
		factory.setValidating(false);
	    //factory.setNamespaceAware(true);
	    SAXParser parser = factory.newSAXParser();
	    myReader = parser.getXMLReader();
		
	}
	
	/**
	 * 
	 * @throws SAXException   XML-
	 * @throws IOException   /
	 */
	public IKnotDiagram read() throws IOException, SAXException {
        
		IKnotDiagram d = new KnotDiagram();
		TheContentHandler ch = new TheContentHandler(d);
		myReader.setContentHandler(ch);
		
		
        InputStream is = new FileInputStream(this.myDiagramFile);
        try {
        	myReader.parse(new InputSource(is));
        	this.myDimension = ch.getDimension();
    		//System.out.println("2 "+this.myDimension);
        } finally {
            is.close();
        }
        
        return d;
	}
	
	public Dimension getDimension(){
		//System.out.println("1 "+this.myDimension);
		return this.myDimension;
	}

	public static String getDescription() {
		return BasicDiagramReader.DESCRIPTION;
	}

	public static Object getFilter() {
		return BasicDiagramReader.FILTER;
	}

}
