package org.beliaj.knots.main;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.image.BufferedImage;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.Border;
import javax.swing.border.LineBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.MouseInputListener;

import org.beliaj.knots.diagram.IKnotDiagram;
import org.beliaj.knots.painters.IEnum;
import org.beliaj.knots.painters.IPainter;
import org.beliaj.knots.painters.PolynomViewer;

/**
 *   .
 * @author Julia
 *
 */
class InternalDiagramFrame extends JInternalFrame{
	
	/**
	 * 
	 */
	private final Main myMainClass;
	
	String myFileName = "";

	/**
	 * 
	 */
	private static final long serialVersionUID = 931962937982635850L;

	/**
	 * .
	 */
	static final int xOffset = 50, yOffset = 50;
	
	MouseInputListener myMouseListener;
	
	/**
	 * ,   .
	 */
	IPainter myPainter;
	PolynomViewer myPolynomViewer;
	
	DiagramPanel panel;

	private boolean hasChanges = false;

	private JLabel myStatusLabel;
	private ChangeListener dcl;
	private class DiagramChangeListener implements ChangeListener{
		
		public void stateChanged(ChangeEvent e) {
			InternalDiagramFrame.this.panel.repaint();
			InternalDiagramFrame.this.hasChanges = true;
			InternalDiagramFrame.this.myMainClass.updateDiagramActions();
			InternalDiagramFrame.this.myMainClass.restoreSettingsMenu();
		}
		
	}
	
	/**
	 * .
	 * @param painter 
	 * @param main TODO
	 * @param dimension 
	 */
	InternalDiagramFrame(Main main, IPainter painter, String title, Dimension dimension){
		super(title,
		          true, //resizable
		          true, //closable
		          true, //maximizable
		          true);
		myMainClass = main;
		this.myPainter = painter;
		myMainClass.myInternalFrames.add(this);
		if (dimension == null){
			this.setBounds(myMainClass.myInset+xOffset*Main.openFrameCount,
				myMainClass.myInset+yOffset*Main.openFrameCount,
				myMainClass.myInset+painter.getWidth(),
				myMainClass.myInset+painter.getHeight());
		} else {
			this.setBounds(myMainClass.myInset+xOffset*Main.openFrameCount,
					myMainClass.myInset+yOffset*Main.openFrameCount,
					dimension.width,
					dimension.height);
		}
		Main.openFrameCount++;
		panel = new DiagramPanel();
		
		dcl = new DiagramChangeListener();
		this.myPainter.getKnotDiagram().addChangeListener(dcl);
		
		this.myMouseListener = this.myPainter.getMouseListener();
		panel.addMouseListener(this.myMouseListener);
		panel.addMouseMotionListener(this.myMouseListener);
		this.add(panel, BorderLayout.CENTER);
		
		panel.addComponentListener(new ComponentAdapter(){
			public void componentResized(ComponentEvent e){
				panel.firstTime = true;
			}
		});
		this.myStatusLabel = new JLabel("");
		this.myStatusLabel.setBorder(new LineBorder(new Color(99, 130, 191)));
		this.add(this.myStatusLabel, BorderLayout.PAGE_END);
		
		this.addFocusListener(new FocusAdapter(){
			public void focusGained(FocusEvent e) {
				InternalDiagramFrame.this.myMainClass.myCurrentDiagramFrame = InternalDiagramFrame.this;
			}
		});
		
		this.myPolynomViewer = new PolynomViewer(this.myPainter.getKnotDiagram(), this.myStatusLabel);
	}
	
	void reloadAllListeners(){
		panel.removeMouseListener(this.myMouseListener);
		panel.removeMouseMotionListener(this.myMouseListener);
		this.myMouseListener = this.myPainter.getMouseListener();
		panel.addMouseListener(this.myMouseListener);
		panel.addMouseMotionListener(this.myMouseListener);
	}
	
	/**
	 * ,   .
	 * @author Julia
	 *
	 */
	public class DiagramPanel extends JPanel{
		
		/**
		 * 
		 */
		private static final long serialVersionUID = -6957028021229012522L;

		/**
		 * 
		 */
		BufferedImage buffImage;
		
		/**
		 *   .
		 */
		Graphics2D buff;

		/**
		 * 
		 */
		boolean firstTime = true;
		
		/**
		 * 
		 */
		public void paint(Graphics g){
			super.paint(g);
			
			Graphics2D g2 = (Graphics2D)g;
			g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
			g2.setStroke(new BasicStroke(3.0f));
			g2.setBackground(Color.white);
			myPainter.repaint(g2);
		}
		
	}

	public IKnotDiagram getKnotDiaram() {
		return this.myPainter.getKnotDiagram();
	}

	public boolean hasChanges() {
		return this.hasChanges ;
	}

	public void setSaved(boolean b) {
		this.hasChanges = !b;
		
	}

	public boolean isActionPossible(IEnum actionCode) {
		return myPainter.isActionPossible(actionCode);
	}

	public void sendChangeEvent(ChangeEvent event) {
		this.dcl.stateChanged(event);
		
	}

	public void viewJonesPolynom() {
		this.myPolynomViewer.viewJonesPolynom();
		this.repaint();
	}
	
}
