package org.beliaj.knots.io;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.LineNumberReader;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import org.beliaj.knots.diagram.IKnotDiagram;
import org.beliaj.knots.diagram.IPoint;
import org.beliaj.knots.diagram.impl.KnotDiagram;
import org.beliaj.knots.io.IDiagramReader;
import org.beliaj.knots.io.IDiagramWriter;
import org.beliaj.knots.io.impl.BasicDiagramReader;
import org.beliaj.knots.io.impl.BasicDiagramWriter;
import org.xml.sax.SAXException;

import junit.framework.AssertionFailedError;
import junit.framework.TestCase;

public class BasicIOTestCase extends TestCase {
	
	/**
	 * 
	 *
	 */
	public void test1(){
		try {
			
			IDiagramReader dr = new BasicDiagramReader("tests/org/beliaj/knots/io/basicDiagram1.knd");
			IKnotDiagram diagram = dr.read();
			TestCase.assertEquals("test1_1",
					"Points:\n" +
					"P0: ( 1, 8 )\n" +
					"P1: ( 6, 10 )\n" +
					"P2: ( 8, 5 )\n" +
					"P3: ( 6, 1 )\n" +
					"P4: ( 8, 12 )\n" +
					"P5: ( 11, 10 )\n" +
					"P6: ( 10, 6 )\n" +
					"P7: ( 4, 5 )\n" +
					"P8: ( 11, 12 )\n" +
					"Lines:\n" +
					"L0: ( P1, P0 )\n" +
					"L1: ( P2, P1 )\n" +
					"    I0 I1 \n" +
					"L2: ( P3, P2 )\n" +
					"L3: ( P8, P7 )\n" +
					"    I0 I2 \n" +
					"L4: ( P7, P6 )\n" +
					"    I1 \n" +
					"L5: ( P6, P5 )\n" +
					"L6: ( P5, P4 )\n" +
					"    I2 \n" +
					"Intersections:\n" +
					"I0 ( L3, L1 )\n" +
					"I1 ( L4, L1 )\n" +
					"I2 ( L6, L3 )\n",
					diagram.toString());
			
		} catch (SAXException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (ParserConfigurationException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (IOException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		}
	}
	
	public static String fromStackTraceToString(StackTraceElement[] stackTrace) {
		StringBuffer sb = new StringBuffer();
		for (StackTraceElement el: stackTrace){
			sb.append(el.toString());
			sb.append("\n");
		}
		return sb.toString();
	}

	/**
	 *  + 
	 *
	 */
	public void test2(){
		IDiagramWriter dw = new BasicDiagramWriter("tests/org/beliaj/knots/io/basicDiagram2.knd");
		
		IKnotDiagram knotDiagram = new KnotDiagram();	
		
		List<IPoint> list = knotDiagram.getPoints();
		
		knotDiagram.addPoint(1, 8);
		knotDiagram.addPoint(6, 10);
		knotDiagram.addPoint(8, 5);
		
		knotDiagram.connectPoints(list.get(0), list.get(1));
		knotDiagram.connectPoints(list.get(1), list.get(2));
		
		knotDiagram.addPoint(6, 1);
		knotDiagram.addPoint(8, 12);
		knotDiagram.addPoint(11, 10);
		
		knotDiagram.connectPoints(list.get(4), list.get(5));
		
		knotDiagram.addPoint(10, 6);
		knotDiagram.addPoint(4, 5);
		knotDiagram.addPoint(11, 12);
		
		
		knotDiagram.connectPoints(list.get(5), list.get(6));
		knotDiagram.connectPoints(list.get(7), list.get(8));
		knotDiagram.connectPoints(list.get(6), list.get(7));
		knotDiagram.connectPoints(list.get(2), list.get(3));
		
		try {
			dw.write(knotDiagram);
		} catch (IOException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
			//e.printStackTrace();
		} catch (ParserConfigurationException e) {
			//e.printStackTrace();
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (TransformerException e) {
			//e.printStackTrace();
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		}
		
		try {
			
			IDiagramReader dr = new BasicDiagramReader("tests/org/beliaj/knots/io/basicDiagram2.knd");
			IKnotDiagram diagram = dr.read();
			TestCase.assertEquals("test2_1", knotDiagram.toString(), diagram.toString());
			
		} catch (SAXException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (ParserConfigurationException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (IOException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		}
		
	}
	
	/**
	 *  + 
	 *
	 */
	public void test3(){
		IDiagramWriter dw = new BasicDiagramWriter("tests/org/beliaj/knots/io/basicDiagram3.knd");
		
		KnotDiagram knotDiagram = new KnotDiagram();
		
		List<IPoint> list = knotDiagram.getPoints();
		
		knotDiagram.addPoint(2, 6);
		knotDiagram.addPoint(8, 6);
		knotDiagram.addPoint(6, 9);
		knotDiagram.addPoint(6, 4);
		knotDiagram.connectPoints(list.get(0), list.get(1));
		knotDiagram.connectPoints(list.get(2), list.get(3));
		knotDiagram.connectPoints(list.get(2), list.get(0));
		
		try {
			dw.write(knotDiagram);
		} catch (IOException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (ParserConfigurationException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (TransformerException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		}
		
		try {
			
			IDiagramReader dr = new BasicDiagramReader("tests/org/beliaj/knots/io/basicDiagram3.knd");
			IKnotDiagram diagram = dr.read();
			TestCase.assertEquals("test3_1", knotDiagram.toString(), diagram.toString());
			
		} catch (SAXException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (ParserConfigurationException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (IOException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		}
	}
	
	/**
	 *  + 
	 *
	 */
	public void test4(){
		IDiagramWriter dw = new BasicDiagramWriter("tests/org/beliaj/knots/io/basicDiagram4.knd");
		
		
		KnotDiagram knotDiagram = new KnotDiagram();
		List<IPoint> list = knotDiagram.getPoints();
		
		knotDiagram.addPoint(2, 1);
		knotDiagram.addPoint(9, 7);
		knotDiagram.addPoint(2, 7);
		knotDiagram.addPoint(9, 1);
		
		knotDiagram.connectPoints(list.get(1),list.get(2));
		knotDiagram.connectPoints(list.get(1),list.get(3));
		knotDiagram.connectPoints(list.get(0),list.get(2));
		knotDiagram.connectPoints(list.get(0),list.get(3));
		
		try {
			dw.write(knotDiagram);
		} catch (IOException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (ParserConfigurationException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (TransformerException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		}
		
		try {
			
			IDiagramReader dr = new BasicDiagramReader("tests/org/beliaj/knots/io/basicDiagram4.knd");
			IKnotDiagram diagram = dr.read();
			TestCase.assertEquals("test4_1", knotDiagram.toString(), diagram.toString());
			
		} catch (SAXException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (ParserConfigurationException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (IOException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		}
	}
	
	/**
	 * 
	 *
	 */
	public void test5(){
		try {
			
			IDiagramReader dr = new BasicDiagramReader("tests/org/beliaj/knots/io/basicDiagram5.knd");
			IKnotDiagram diagram = dr.read();
			TestCase.assertEquals("test5_1",
					"Points:\n" +
					"P0: ( 1, 8 )\n" +
					"P1: ( 6, 10 )\n" +
					"P2: ( 8, 5 )\n" +
					"P3: ( 6, 1 )\n" +
					"P4: ( 8, 12 )\n" +
					"P5: ( 11, 10 )\n" +
					"P6: ( 10, 6 )\n" +
					"P7: ( 4, 5 )\n" +
					"P8: ( 11, 12 )\n" +
					"Lines:\n" +
					"L0: ( P1, P0 )\n" +
					"L1: ( P2, P1 )\n" +
					"    I0 I1 \n" +
					"L2: ( P3, P2 )\n" +
					"L3: ( P8, P7 )\n" +
					"    I0 I2 \n" +
					"L4: ( P7, P6 )\n" +
					"    I1 \n" +
					"L5: ( P6, P5 )\n" +
					"L6: ( P5, P4 )\n" +
					"    I2 \n" +
					"Intersections:\n" +
					"I1 ( L1, L4 )\n" +
					"I0 ( L3, L1 )\n" +
					"I2 ( L3, L6 )\n",
					diagram.toString());
			
		} catch (SAXException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (ParserConfigurationException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (IOException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		}
	}
	
	/**
	 * 
	 *
	 */
	public void test6(){
		IDiagramWriter dw = new BasicDiagramWriter("tests/org/beliaj/knots/io/basicDiagram6.knd");
		
		IKnotDiagram knotDiagram = new KnotDiagram();	
		
		List<IPoint> list = knotDiagram.getPoints();
		
		knotDiagram.addPoint(1, 8);
		knotDiagram.addPoint(6, 10);
		knotDiagram.addPoint(8, 5);
		
		knotDiagram.connectPoints(list.get(0), list.get(1));
		knotDiagram.connectPoints(list.get(1), list.get(2));
		
		knotDiagram.addPoint(6, 1);
		knotDiagram.addPoint(8, 12);
		knotDiagram.addPoint(11, 10);
		
		knotDiagram.connectPoints(list.get(4), list.get(5));
		
		knotDiagram.addPoint(10, 6);
		knotDiagram.addPoint(4, 5);
		knotDiagram.addPoint(11, 12);
		
		
		knotDiagram.connectPoints(list.get(5), list.get(6));
		knotDiagram.connectPoints(list.get(7), list.get(8));
		knotDiagram.connectPoints(list.get(6), list.get(7));
		knotDiagram.connectPoints(list.get(2), list.get(3));
		
		try {
			dw.write(knotDiagram);
		} catch (IOException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (ParserConfigurationException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (TransformerException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		}
		
		try {
			TestCase.assertTrue(sameFiles("tests/org/beliaj/knots/io/basicDiagram6.knd", "tests/org/beliaj/knots/io/basicDiagramEx6.knd"));
		} catch (FileNotFoundException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (IOException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		}
		
	}
	
	private boolean sameFiles(String string, String string2) throws IOException{
		FileReader reader1 = new FileReader( new File(string));
		FileReader reader2 = new FileReader( new File(string2));
		
		LineNumberReader lnr1 = new LineNumberReader(reader1);
		LineNumberReader lnr2 = new LineNumberReader(reader2);
		
		String str = "";
		String str1 = "";
		while ((str = lnr1.readLine()) != null) {
			str1 = lnr2.readLine();
			if ( (str1 == null) || (!str1.equals(str))){
				throw new AssertionFailedError("" + str1 + " != " + str);
			}
		}
		
		if (lnr2.read() >= 0){
			throw new AssertionFailedError("lnr2.read() >= 0");
		}
		
		return true;
	}

	/**
	 * 
	 *
	 */
	public void test7(){
		IDiagramWriter dw = new BasicDiagramWriter("tests/org/beliaj/knots/io/basicDiagram7.knd");
		
		KnotDiagram knotDiagram = new KnotDiagram();
		
		List<IPoint> list = knotDiagram.getPoints();
		
		knotDiagram.addPoint(2, 6);
		knotDiagram.addPoint(8, 6);
		knotDiagram.addPoint(6, 9);
		knotDiagram.addPoint(6, 4);
		knotDiagram.connectPoints(list.get(0), list.get(1));
		knotDiagram.connectPoints(list.get(2), list.get(3));
		knotDiagram.connectPoints(list.get(0), list.get(2));
		
		try {
			dw.write(knotDiagram);
		} catch (IOException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (ParserConfigurationException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (TransformerException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		}
	
		try {
			TestCase.assertTrue(sameFiles("tests/org/beliaj/knots/io/basicDiagram7.knd", "tests/org/beliaj/knots/io/basicDiagramEx7.knd"));
		} catch (IOException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		}
		
	}
	
	/**
	 * 
	 *
	 */
	public void test8(){
		IDiagramWriter dw = new BasicDiagramWriter("tests/org/beliaj/knots/io/basicDiagram8.knd");
		
		
		KnotDiagram knotDiagram = new KnotDiagram();
		List<IPoint> list = knotDiagram.getPoints();
		
		knotDiagram.addPoint(2, 1);
		knotDiagram.addPoint(9, 7);
		knotDiagram.addPoint(2, 7);
		knotDiagram.addPoint(9, 1);
		
		knotDiagram.connectPoints(list.get(1),list.get(2));
		knotDiagram.connectPoints(list.get(1),list.get(3));
		knotDiagram.connectPoints(list.get(0),list.get(2));
		knotDiagram.connectPoints(list.get(0),list.get(3));
		
		try {
			dw.write(knotDiagram);
		} catch (IOException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (ParserConfigurationException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (TransformerException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		}
		
		
		try {
			TestCase.assertTrue(sameFiles("tests/org/beliaj/knots/io/basicDiagram8.knd",
					"tests/org/beliaj/knots/io/basicDiagramEx8.knd"));
		} catch (IOException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		}
		
	}
	
	/**
	 * 
	 *
	 */
	public void test9(){
		IDiagramWriter dw = new BasicDiagramWriter("tests/org/beliaj/knots/io/basicDiagram9.knd");
		
		
		KnotDiagram knotDiagram = new KnotDiagram();
		
		try {
			dw.write(knotDiagram);
		} catch (IOException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (ParserConfigurationException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (TransformerException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		}
		
		
		try {
			TestCase.assertTrue(sameFiles("tests/org/beliaj/knots/io/basicDiagram9.knd",
					"tests/org/beliaj/knots/io/basicDiagramEx9.knd"));
		} catch (IOException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		}
		
	}
	
	/**
	 * 
	 *
	 */
	public void test10(){
			
			IDiagramReader dr;
			try {
				dr = new BasicDiagramReader("tests/org/beliaj/knots/io/basicDiagram9.knd");
				IKnotDiagram diagram = dr.read();
				
				IKnotDiagram diagramEx = new KnotDiagram();
				
				TestCase.assertEquals("test10_1:",
						diagram.toString(), diagramEx.toString());
			} catch (SAXException e) {
				throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
			} catch (ParserConfigurationException e) {
				throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
			} catch (IOException e) {
				throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
			}
			
	}
	
	/**
	 * 
	 *
	 */
	public void test11(){
		try {
			
			IDiagramReader dr = new BasicDiagramReader("tests/org/beliaj/knots/io/basicDiagram11.knd");
			IKnotDiagram diagram = dr.read();
			TestCase.assertEquals("test1_1",
					"Points:\n" +
					"P0: ( 8, 12 )\n" +
					"P1: ( 6, 10 )\n" +
					"P2: ( 8, 5 )\n" +
					"P3: ( 6, 1 )\n" +
					"P4: ( 10, 6 )\n" +
					"P5: ( 4, 5 )\n" +
					"P6: ( 11, 12 )\n" +
					"P7: ( 11, 10 )\n" +
					"P8: ( 1, 8 )\n" +
					"Lines:\n" +
					"L0: ( P1, P8 )\n" +
					"L1: ( P2, P1 )\n" +
					"    I0 I1 \n" +
					"L2: ( P3, P2 )\n" +
					"L3: ( P6, P5 )\n" +
					"    I0 I2 \n" +
					"L4: ( P5, P4 )\n" +
					"    I1 \n" +
					"L5: ( P4, P7 )\n" +
					"L6: ( P7, P0 )\n" +
					"    I2 \n" +
					"Intersections:\n" +
					"I0 ( L3, L1 )\n" +
					"I1 ( L4, L1 )\n" +
					"I2 ( L6, L3 )\n",
					diagram.toString());
			
		} catch (SAXException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (ParserConfigurationException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (IOException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		}
	}
	
	/**
	 * 
	 *
	 */
	public void test12(){
		IKnotDiagram knotDiagram = new KnotDiagram();
		
		List<IPoint> list = knotDiagram.getPoints();
		
		knotDiagram.addPoint(3, 5);
		knotDiagram.addPoint(-5, -2);
		knotDiagram.addPoint(-2, 5);
		knotDiagram.addPoint(0, 6);
		knotDiagram.addPoint(5, 1);
		knotDiagram.addPoint(6, -2);
		knotDiagram.addPoint(4, -3);
		knotDiagram.addPoint(-2, -2);
		
		knotDiagram.connectPoints(list.get(0), list.get(1));
		knotDiagram.connectPoints(list.get(1), list.get(6));
		knotDiagram.connectPoints(list.get(2), list.get(5));
		knotDiagram.connectPoints(list.get(3), list.get(7));
		knotDiagram.connectPoints(list.get(4), list.get(7));
		knotDiagram.connectPoints(list.get(3), list.get(4));
		
		IDiagramWriter dw = new BasicDiagramWriter("tests/org/beliaj/knots/io/basicDiagram12.knd");
		
		try {
			dw.write(knotDiagram);
		} catch (IOException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (ParserConfigurationException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (TransformerException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		}
		
		try {
			TestCase.assertTrue(sameFiles("tests/org/beliaj/knots/io/basicDiagram12.knd",
					"tests/org/beliaj/knots/io/basicDiagramEx12.knd"));
		} catch (IOException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		}
		
	}
	
	/**
	 *  
	 *
	 */
	public void test13(){
		IKnotDiagram knotDiagram = new KnotDiagram();
		
		List<IPoint> list = knotDiagram.getPoints();
		
		knotDiagram.addPoint(2, 2);
		knotDiagram.addPoint(3, 2);
		knotDiagram.addPoint(3, 3);
		knotDiagram.addPoint(3, 4);
		knotDiagram.addPoint(4, 4);
		knotDiagram.addPoint(4, 3);
		knotDiagram.addPoint(4, 2);
		knotDiagram.addPoint(5, 2);
		knotDiagram.addPoint(5, 4);
		knotDiagram.addPoint(5, 7);
		knotDiagram.addPoint(5, 9);
		knotDiagram.addPoint(3, 9);
		knotDiagram.addPoint(2, 9);
		knotDiagram.addPoint(3, 8);
		knotDiagram.addPoint(4, 8);
		knotDiagram.addPoint(3, 7);
		knotDiagram.addPoint(4, 7);
		knotDiagram.addPoint(2, 7);
		knotDiagram.addPoint(2, 4);
		knotDiagram.addPoint(4, 9);
		
		knotDiagram.connectPoints(list.get(0), list.get(1));
		knotDiagram.connectPoints(list.get(6), list.get(7));
		knotDiagram.connectPoints(list.get(12), list.get(11));
		knotDiagram.connectPoints(list.get(19), list.get(10));
		knotDiagram.connectPoints(list.get(17), list.get(4));
		knotDiagram.connectPoints(list.get(15), list.get(8));
		knotDiagram.connectPoints(list.get(16), list.get(18));
		knotDiagram.connectPoints(list.get(9), list.get(3));
		knotDiagram.connectPoints(list.get(11), list.get(14));
		knotDiagram.connectPoints(list.get(19), list.get(13));
		knotDiagram.connectPoints(list.get(2), list.get(6));
		knotDiagram.connectPoints(list.get(5), list.get(1));
		knotDiagram.connectPoints(list.get(0), list.get(18));
		knotDiagram.connectPoints(list.get(12), list.get(17));
		knotDiagram.connectPoints(list.get(13), list.get(15));
		knotDiagram.connectPoints(list.get(16), list.get(14));
		knotDiagram.connectPoints(list.get(3), list.get(2));
		knotDiagram.connectPoints(list.get(5), list.get(4));
		knotDiagram.connectPoints(list.get(8), list.get(7));
		knotDiagram.connectPoints(list.get(9), list.get(10));
		
		knotDiagram.setIntersectionType(knotDiagram.getLines().get(4), knotDiagram.getLines().get(6));
		knotDiagram.setIntersectionType(knotDiagram.getLines().get(4), knotDiagram.getLines().get(7));
		knotDiagram.setIntersectionType(knotDiagram.getLines().get(6), knotDiagram.getLines().get(5));
		
		knotDiagram.setIntersectionType(knotDiagram.getLines().get(5), knotDiagram.getLines().get(7));
		knotDiagram.setIntersectionType(knotDiagram.getLines().get(10), knotDiagram.getLines().get(11));
		knotDiagram.setIntersectionType(knotDiagram.getLines().get(9), knotDiagram.getLines().get(8));
		
		IDiagramWriter dw = new BasicDiagramWriter("tests/org/beliaj/knots/io/basicDiagram13.knd");
		
		try {
			dw.write(knotDiagram);
		} catch (IOException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (ParserConfigurationException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (TransformerException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		}
		
		try {
			TestCase.assertTrue(sameFiles("tests/org/beliaj/knots/io/basicDiagram13.knd",
					"tests/org/beliaj/knots/io/basicDiagramEx13.knd"));
		} catch (IOException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		}
	}
	
	/**
	 *  
	 *
	 */
	public void test14(){
		try {
			
			IDiagramReader dr = new BasicDiagramReader("tests/org/beliaj/knots/io/basicDiagram14.knd");
			IKnotDiagram diagram = dr.read();
			TestCase.assertEquals("test14_1",
					"Points:\n" +
					"P0: ( 2, 4 )\n" +
					"P1: ( 2, 2 )\n" +
					"P2: ( 3, 2 )\n" +
					"P3: ( 3, 3 )\n" +
					"P4: ( 5, 7 )\n" +
					"P5: ( 4, 4 )\n" +
					"P6: ( 4, 3 )\n" +
					"P7: ( 4, 2 )\n" +
					"P8: ( 5, 2 )\n" +
					"P9: ( 5, 4 )\n" +
					"P10: ( 4, 9 )\n" +
					"P11: ( 5, 9 )\n" +
					"P12: ( 3, 9 )\n" +
					"P13: ( 2, 9 )\n" +
					"P14: ( 3, 8 )\n" +
					"P15: ( 3, 4 )\n" +
					"P16: ( 4, 8 )\n" +
					"P17: ( 3, 7 )\n" +
					"P18: ( 4, 7 )\n" +
					"P19: ( 2, 7 )\n" +
					"Lines:\n" +
					"L0: ( P11, P10 )\n" +
					"L1: ( P2, P1 )\n" +
					"L2: ( P8, P7 )\n" +
					"L3: ( P12, P13 )\n" +
					"L4: ( P19, P5 )\n" +
					"    I0 I2 \n" +
					"L5: ( P17, P9 )\n" +
					"    I1 I3 \n" +
					"L6: ( P1, P0 )\n" +
					"L7: ( P0, P18 )\n" +
					"    I0 I1 \n" +
					"L8: ( P15, P4 )\n" +
					"    I2 I3 \n" +
					"L9: ( P16, P12 )\n" +
					"    I4 \n" +
					"L10: ( P10, P14 )\n" +
					"    I4 \n" +
					"L11: ( P7, P3 )\n" +
					"    I5 \n" +
					"L12: ( P6, P2 )\n" +
					"    I5 \n" +
					"L13: ( P13, P19 )\n" +
					"L14: ( P14, P17 )\n" +
					"L15: ( P18, P16 )\n" +
					"L16: ( P3, P15 )\n" +
					"L17: ( P9, P8 )\n" +
					"L18: ( P4, P11 )\n" +
					"L19: ( P5, P6 )\n" +
					"Intersections:\n" +
					"I0 ( L4, L7 )\n" +
					"I2 ( L4, L8 )\n" +
					"I3 ( L5, L8 )\n" +
					"I1 ( L7, L5 )\n" +
					"I4 ( L10, L9 )\n" +
					"I5 ( L11, L12 )\n",
					diagram.toString());
			
		} catch (SAXException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (ParserConfigurationException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		} catch (IOException e) {
			throw new AssertionFailedError(fromStackTraceToString(e.getStackTrace()));
		}
	}
}
