package org.beliaj.knots.simple;

import java.io.IOException;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;

import org.beliaj.knots.diagram.IKnotDiagram;
import org.beliaj.knots.diagram.IPoint;
import org.beliaj.knots.diagram.impl.KnotDiagram;
import org.beliaj.knots.io.BasicIOTestCase;
import org.beliaj.knots.io.IDiagramReader;
import org.beliaj.knots.io.impl.BasicDiagramReader;
import org.beliaj.knots.math.Polynom;
import org.beliaj.knots.simple.SimpleKnot;
import org.xml.sax.SAXException;

import junit.framework.AssertionFailedError;
import junit.framework.TestCase;

public class SimpleKnotTestCase extends TestCase {

	public void test1(){
		IKnotDiagram knotDiagram = new KnotDiagram();
		
		List<IPoint> list = knotDiagram.getPoints();
		
		knotDiagram.addPoint(1, 1);
		knotDiagram.addPoint(1, 6);
		knotDiagram.addPoint(6, 1);
		
		knotDiagram.connectPoints(list.get(0), list.get(1));
		knotDiagram.connectPoints(list.get(1), list.get(2));
		knotDiagram.connectPoints(list.get(2), list.get(0));
		
		SimpleKnot sk = new SimpleKnot(knotDiagram);
		
		//System.out.println(sk);
		TestCase.assertEquals("test1_1", "1 * x^0", sk.getKaufman().toString());
		TestCase.assertEquals("test1_2", "1 * x^0", sk.getJonesKaufman().toString());
		TestCase.assertEquals("test1_3", "1", sk.getJonesInHTML());
	}
	
	public void test2(){
		IKnotDiagram knotDiagram = new KnotDiagram();
		
		List<IPoint> list = knotDiagram.getPoints();
		
		knotDiagram.addPoint(1, 1);
		knotDiagram.addPoint(1, 6);
		knotDiagram.addPoint(6, 1);
		
		knotDiagram.connectPoints(list.get(0), list.get(1));
		knotDiagram.connectPoints(list.get(1), list.get(2));
		knotDiagram.connectPoints(list.get(2), list.get(0));
		
		knotDiagram.addPoint(2, 2);
		knotDiagram.addPoint(2, 4);
		knotDiagram.addPoint(4, 2);
		knotDiagram.addPoint(4, 4);
		
		knotDiagram.connectPoints(list.get(3), list.get(4));
		knotDiagram.connectPoints(list.get(4), list.get(6));
		knotDiagram.connectPoints(list.get(6), list.get(5));
		knotDiagram.connectPoints(list.get(5), list.get(3));
		
		TestCase.assertEquals("test2_1", "Points:\n" +
				"P0: ( 1, 1 )\n" +
				"P1: ( 1, 6 )\n" +
				"P2: ( 6, 1 )\n" +
				"P3: ( 2, 2 )\n" +
				"P4: ( 2, 4 )\n" +
				"P5: ( 4, 2 )\n" +
				"P6: ( 4, 4 )\n" +
				"Lines:\n" +
				"L0: ( P1, P0 )\n" +
				"L1: ( P2, P1 )\n" +
				"    I0 I1 \n" +
				"L2: ( P0, P2 )\n" +
				"L3: ( P4, P3 )\n" +
				"L4: ( P6, P4 )\n" +
				"    I0 \n" +
				"L5: ( P5, P6 )\n" +
				"    I1 \n" +
				"L6: ( P3, P5 )\n" +
				"Intersections:\n" +
				"I0 ( L1, L4 )\n" +
				"I1 ( L1, L5 )\n", knotDiagram.toString());
		
		SimpleKnot sk = new SimpleKnot(knotDiagram);
		TestCase.assertEquals("test2_2", "- 1 * x^-2 - 1 * x^2", sk.getKaufman().toString());
		TestCase.assertEquals("test2_3", "- 1 * x^-2 - 1 * x^2", sk.getJonesKaufman().toString());
		TestCase.assertEquals("test2_4", "- x<sup>1/2</sup> - x<sup>-1/2</sup>", sk.getJonesInHTML());
	}
	
	//  
	public void test3(){
		IKnotDiagram knotDiagram = new KnotDiagram();
		
		List<IPoint> list = knotDiagram.getPoints();
		
		knotDiagram.addPoint(1, 1);
		knotDiagram.addPoint(1, 6);
		knotDiagram.addPoint(6, 1);
		
		knotDiagram.connectPoints(list.get(0), list.get(1));
		knotDiagram.connectPoints(list.get(1), list.get(2));
		knotDiagram.connectPoints(list.get(2), list.get(0));
		
		knotDiagram.addPoint(2, 2);
		knotDiagram.addPoint(2, 4);
		knotDiagram.addPoint(4, 2);
		knotDiagram.addPoint(4, 4);
		
		knotDiagram.connectPoints(list.get(3), list.get(4));
		knotDiagram.connectPoints(list.get(4), list.get(6));
		knotDiagram.connectPoints(list.get(6), list.get(5));
		knotDiagram.connectPoints(list.get(5), list.get(3));
		
		TestCase.assertEquals("test3_1", "Points:\n" +
				"P0: ( 1, 1 )\n" +
				"P1: ( 1, 6 )\n" +
				"P2: ( 6, 1 )\n" +
				"P3: ( 2, 2 )\n" +
				"P4: ( 2, 4 )\n" +
				"P5: ( 4, 2 )\n" +
				"P6: ( 4, 4 )\n" +
				"Lines:\n" +
				"L0: ( P1, P0 )\n" +
				"L1: ( P2, P1 )\n" +
				"    I0 I1 \n" +
				"L2: ( P0, P2 )\n" +
				"L3: ( P4, P3 )\n" +
				"L4: ( P6, P4 )\n" +
				"    I0 \n" +
				"L5: ( P5, P6 )\n" +
				"    I1 \n" +
				"L6: ( P3, P5 )\n" +
				"Intersections:\n" +
				"I0 ( L1, L4 )\n" +
				"I1 ( L1, L5 )\n", knotDiagram.toString());
		knotDiagram.setIntersectionType(knotDiagram.getLines().get(4), knotDiagram.getLines().get(1));
		SimpleKnot sk = new SimpleKnot(knotDiagram);
		//System.out.println(sk);
		TestCase.assertEquals("test3_2", "- 1 * x^-4 - 1 * x^4", sk.getKaufman().toString());
		//TestCase.assertEquals("test2_1", "- 1 * x^-2 - 1 * x^2", sk.getKaufman().toString());
		
		TestCase.assertEquals("test3_3", "- 1 * x^2 - 1 * x^10", sk.getJonesKaufman().toString());
		TestCase.assertEquals("test3_4", "- x<sup>-1/2</sup> - x<sup>-5/2</sup>", sk.getJonesInHTML());
	}
	
	//	
	public void test4(){
		IKnotDiagram knotDiagram = new KnotDiagram();
		
		List<IPoint> list = knotDiagram.getPoints();
		
		knotDiagram.addPoint(1, 1);
		knotDiagram.addPoint(6, 2);
		knotDiagram.addPoint(7, 4);
		knotDiagram.addPoint(7, 7);
		knotDiagram.addPoint(2, 6);
		knotDiagram.addPoint(4, 1);
		knotDiagram.addPoint(7, 1);
		knotDiagram.addPoint(6, 6);
		knotDiagram.addPoint(1, 7);

		knotDiagram.connectPoints(list.get(0), list.get(8));
		knotDiagram.connectPoints(list.get(0), list.get(1));
		knotDiagram.connectPoints(list.get(6), list.get(7));
		knotDiagram.connectPoints(list.get(1), list.get(2));
		knotDiagram.connectPoints(list.get(2), list.get(3));
		knotDiagram.connectPoints(list.get(3), list.get(4));
		knotDiagram.connectPoints(list.get(7), list.get(8));
		knotDiagram.connectPoints(list.get(6), list.get(5));
		knotDiagram.connectPoints(list.get(5), list.get(4));
		
		TestCase.assertEquals("test4_1", "Points:\n" +
				"P0: ( 1, 1 )\n" +
				"P1: ( 6, 2 )\n" +
				"P2: ( 7, 4 )\n" +
				"P3: ( 7, 7 )\n" +
				"P4: ( 2, 6 )\n" +
				"P5: ( 4, 1 )\n" +
				"P6: ( 7, 1 )\n" +
				"P7: ( 6, 6 )\n" +
				"P8: ( 1, 7 )\n" +
				"Lines:\n" +
				"L0: ( P0, P8 )\n" +
				"L1: ( P1, P0 )\n" +
				"    I2 \n" +
				"L2: ( P7, P6 )\n" +
				"    I0 \n" +
				"L3: ( P2, P1 )\n" +
				"    I0 \n" +
				"L4: ( P3, P2 )\n" +
				"L5: ( P4, P3 )\n" +
				"    I1 \n" +
				"L6: ( P8, P7 )\n" +
				"    I1 \n" +
				"L7: ( P6, P5 )\n" +
				"L8: ( P5, P4 )\n" +
				"    I2 \n" +
				"Intersections:\n" +
				"I2 ( L1, L8 )\n" +
				"I0 ( L2, L3 )\n" +
				"I1 ( L5, L6 )\n", knotDiagram.toString());
		//System.out.println(knotDiagram);
		knotDiagram.setIntersectionType(knotDiagram.getLines().get(1), knotDiagram.getLines().get(8));
		knotDiagram.setIntersectionType(knotDiagram.getLines().get(2), knotDiagram.getLines().get(3));
		knotDiagram.setIntersectionType(knotDiagram.getLines().get(5), knotDiagram.getLines().get(6));
		SimpleKnot sk = new SimpleKnot(knotDiagram);
		//System.out.println(sk);
		TestCase.assertEquals("test4_2", "- 1 * x^-5 - 1 * x^3 + 1 * x^7", sk.getKaufman().toString());
		//TestCase.assertEquals("test2_1", "- 1 * x^-2 - 1 * x^2", sk.getKaufman().toString());
		
		TestCase.assertEquals("test4_3", "1 * x^4 + 1 * x^12 - 1 * x^16", sk.getJonesKaufman().toString());
		TestCase.assertEquals("test4_4", "x<sup>-1</sup> + x<sup>-3</sup> - x<sup>-4</sup>", sk.getJonesInHTML());
	}
	
	public void test5(){
		IKnotDiagram diagram = null;
		try {
			
			IDiagramReader dr = new BasicDiagramReader("tests/org/beliaj/knots/simple/knotDiagramEx5.knd");
			diagram = dr.read();
		} catch (SAXException e) {
			throw new AssertionFailedError(BasicIOTestCase.fromStackTraceToString(e.getStackTrace()));
		} catch (ParserConfigurationException e) {
			throw new AssertionFailedError(BasicIOTestCase.fromStackTraceToString(e.getStackTrace()));
		} catch (IOException e) {
			throw new AssertionFailedError(BasicIOTestCase.fromStackTraceToString(e.getStackTrace()));
		}
		
		SimpleKnot sk = new SimpleKnot(diagram);
		TestCase.assertEquals("test5_1", "- 1 * x^-3",sk.getKaufman().toString());
		TestCase.assertEquals("test5_2", "1 * x^0",sk.getJonesKaufman().toString());
		TestCase.assertEquals("test5_3", "1", sk.getJonesInHTML());
	}
	
	public void test6(){
		IKnotDiagram diagram = null;
		try {
			
			IDiagramReader dr = new BasicDiagramReader("tests/org/beliaj/knots/simple/knotDiagram6.knd");
			diagram = dr.read();
		} catch (SAXException e) {
			throw new AssertionFailedError(BasicIOTestCase.fromStackTraceToString(e.getStackTrace()));
		} catch (ParserConfigurationException e) {
			throw new AssertionFailedError(BasicIOTestCase.fromStackTraceToString(e.getStackTrace()));
		} catch (IOException e) {
			throw new AssertionFailedError(BasicIOTestCase.fromStackTraceToString(e.getStackTrace()));
		}
		
		SimpleKnot sk = new SimpleKnot(diagram);
		TestCase.assertEquals("test6_1", "- 1 * x^-3",sk.getKaufman().toString());
		TestCase.assertEquals("test6_2", "1 * x^0",sk.getJonesKaufman().toString());
		TestCase.assertEquals("test6_3", "1", sk.getJonesInHTML());
	}
	
	public void test7(){
		IKnotDiagram diagram = null;
		try {
			
			IDiagramReader dr = new BasicDiagramReader("tests/org/beliaj/knots/simple/knotDiagram7.knd");
			diagram = dr.read();
		} catch (SAXException e) {
			throw new AssertionFailedError(BasicIOTestCase.fromStackTraceToString(e.getStackTrace()));
		} catch (ParserConfigurationException e) {
			throw new AssertionFailedError(BasicIOTestCase.fromStackTraceToString(e.getStackTrace()));
		} catch (IOException e) {
			throw new AssertionFailedError(BasicIOTestCase.fromStackTraceToString(e.getStackTrace()));
		}
		
		SimpleKnot sk = new SimpleKnot(diagram);
		TestCase.assertEquals("test7_1", "- 1 * x^-2 - 1 * x^2",sk.getKaufman().toString());
		TestCase.assertEquals("test7_2", "- 1 * x^-2 - 1 * x^2",sk.getJonesKaufman().toString());
		TestCase.assertEquals("test7_3", "- x<sup>1/2</sup> - x<sup>-1/2</sup>", sk.getJonesInHTML());
	}
	
	public void test8(){
		IKnotDiagram diagram = null;
		try{
			SimpleKnot sk = new SimpleKnot(diagram);
			TestCase.fail("There must be null pointer exception!");
		} catch (NullPointerException ne){
			
		}
		
	}
	
	public void test9(){
		IKnotDiagram diagram = new KnotDiagram();
		try{
			SimpleKnot sk = new SimpleKnot(diagram);
			TestCase.fail("There must be runtime exception!");
		} catch (RuntimeException ne){
			
		}
		
	}
	
	public void test10(){
		
		IKnotDiagram knotDiagram = new KnotDiagram();
		
		List<IPoint> list = knotDiagram.getPoints();
		
		knotDiagram.addPoint(1, 8);
		knotDiagram.addPoint(6, 10);
		knotDiagram.addPoint(8, 5);
		
		knotDiagram.connectPoints(list.get(0), list.get(1));
		knotDiagram.connectPoints(list.get(1), list.get(2));
		knotDiagram.addPoint(6, 1);
		knotDiagram.addPoint(8, 12);
		knotDiagram.addPoint(11, 10);
		
		knotDiagram.connectPoints(list.get(4), list.get(5));
		knotDiagram.addPoint(10, 6);
		knotDiagram.addPoint(4, 5);
		knotDiagram.addPoint(11, 12);
		
		knotDiagram.connectPoints(list.get(5), list.get(6));
		knotDiagram.connectPoints(list.get(7), list.get(8));
		knotDiagram.connectPoints(list.get(6), list.get(7));
		knotDiagram.connectPoints(list.get(2), list.get(3));
		
		try{
			SimpleKnot sk = new SimpleKnot(knotDiagram);
			TestCase.fail("There must be runtime exception!");
		} catch (RuntimeException ne){
			
		}
	}
	
	public void test11(){
		IKnotDiagram diagram1 = null;
		try {
			
			IDiagramReader dr1 = new BasicDiagramReader("tests/org/beliaj/knots/simple/big1.knd");
			diagram1 = dr1.read();
		} catch (SAXException e) {
			throw new AssertionFailedError(BasicIOTestCase.fromStackTraceToString(e.getStackTrace()));
		} catch (ParserConfigurationException e) {
			throw new AssertionFailedError(BasicIOTestCase.fromStackTraceToString(e.getStackTrace()));
		} catch (IOException e) {
			throw new AssertionFailedError(BasicIOTestCase.fromStackTraceToString(e.getStackTrace()));
		}
		
		SimpleKnot sk1 = new SimpleKnot(diagram1);
		Polynom kaufman1 = sk1.getKaufman();
		Polynom jonesKaufman1 = sk1.getJonesKaufman();
		TestCase.assertEquals("test11_1",
				"1 * x^-19 - 2 * x^-15 + 2 * x^-11 - 2 * x^-7 - 1 * x^5 + 2 * x^9 - 2 * x^13 + 2 * x^17 - 1 * x^21", 
				kaufman1.toString());
		TestCase.assertEquals("test11_2",
				"- 1 * x^-16 + 2 * x^-12 - 2 * x^-8 + 2 * x^-4 + 1 * x^8 - 2 * x^12 + 2 * x^16 - 2 * x^20 + 1 * x^24",
				jonesKaufman1.toString());
		TestCase.assertEquals("test11_3",
				"- x<sup>4</sup> + 2x<sup>3</sup> - 2x<sup>2</sup> + 2x<sup>1</sup> + x<sup>-2</sup> - 2x<sup>-3</sup> + 2x<sup>-4</sup> - 2x<sup>-5</sup> + x<sup>-6</sup>",
				sk1.getJonesInHTML());
		
		IKnotDiagram diagram2 = null;
		try {
			
			IDiagramReader dr2 = new BasicDiagramReader("tests/org/beliaj/knots/simple/big2.knd");
			diagram2 = dr2.read();
		} catch (SAXException e) {
			throw new AssertionFailedError(BasicIOTestCase.fromStackTraceToString(e.getStackTrace()));
		} catch (ParserConfigurationException e) {
			throw new AssertionFailedError(BasicIOTestCase.fromStackTraceToString(e.getStackTrace()));
		} catch (IOException e) {
			throw new AssertionFailedError(BasicIOTestCase.fromStackTraceToString(e.getStackTrace()));
		}
		
		SimpleKnot sk2 = new SimpleKnot(diagram2);
		Polynom kaufman2 = sk2.getKaufman();
		Polynom jonesKaufman2 = sk2.getJonesKaufman();
		TestCase.assertEquals("test11_1",
				"1 * x^-19 - 2 * x^-15 + 2 * x^-11 - 2 * x^-7 - 1 * x^5 + 2 * x^9 - 2 * x^13 + 2 * x^17 - 1 * x^21", 
				kaufman2.toString());
		TestCase.assertEquals("test11_2",
				"- 1 * x^-16 + 2 * x^-12 - 2 * x^-8 + 2 * x^-4 + 1 * x^8 - 2 * x^12 + 2 * x^16 - 2 * x^20 + 1 * x^24",
				jonesKaufman2.toString());
		TestCase.assertEquals("test11_6",
				"- x<sup>4</sup> + 2x<sup>3</sup> - 2x<sup>2</sup> + 2x<sup>1</sup> + x<sup>-2</sup> - 2x<sup>-3</sup> + 2x<sup>-4</sup> - 2x<sup>-5</sup> + x<sup>-6</sup>",
				sk2.getJonesInHTML());
		
		TestCase.assertEquals("test11_7", kaufman1.toString(), kaufman2.toString());
		TestCase.assertEquals("test11_8", jonesKaufman2.toString(), jonesKaufman2.toString());
		TestCase.assertEquals("test11_9", sk1.getJonesInHTML(), sk2.getJonesInHTML());
	}
	
	public void test12(){
		IKnotDiagram diagram1 = null;
		try {
			
			IDiagramReader dr1 = new BasicDiagramReader("tests/org/beliaj/knots/simple/sum1.knd");
			diagram1 = dr1.read();
		} catch (SAXException e) {
			throw new AssertionFailedError(BasicIOTestCase.fromStackTraceToString(e.getStackTrace()));
		} catch (ParserConfigurationException e) {
			throw new AssertionFailedError(BasicIOTestCase.fromStackTraceToString(e.getStackTrace()));
		} catch (IOException e) {
			throw new AssertionFailedError(BasicIOTestCase.fromStackTraceToString(e.getStackTrace()));
		}
		
		SimpleKnot sk1 = new SimpleKnot(diagram1);
		Polynom kaufman1 = sk1.getKaufman();
		Polynom jonesKaufman1 = sk1.getJonesKaufman();
		TestCase.assertEquals("test12_1",
				"1 * x^-17 - 1 * x^-13 + 3 * x^-9 - 4 * x^-5 + 5 * x^-1 - 5 * x^3 + 5 * x^7 - 3 * x^11 + 2 * x^15 - 1 * x^19",
				kaufman1.toString());
		TestCase.assertEquals("test12_2",
				"- 1 * x^-14 + 1 * x^-10 - 3 * x^-6 + 4 * x^-2 - 5 * x^2 + 5 * x^6 - 5 * x^10 + 3 * x^14 - 2 * x^18 + 1 * x^22",
				jonesKaufman1.toString());
		TestCase.assertEquals("test12_3",
				"- x<sup>7/2</sup> + x<sup>5/2</sup> - 3x<sup>3/2</sup> + 4x<sup>1/2</sup> - 5x<sup>-1/2</sup> + 5x<sup>-3/2</sup> - 5x<sup>-5/2</sup> + 3x<sup>-7/2</sup> - 2x<sup>-9/2</sup> + x<sup>-11/2</sup>",
				sk1.getJonesInHTML());
		
		IKnotDiagram diagram2 = null;
		try {
			
			IDiagramReader dr2 = new BasicDiagramReader("tests/org/beliaj/knots/simple/sum2.knd");
			diagram2 = dr2.read();
		} catch (SAXException e) {
			throw new AssertionFailedError(BasicIOTestCase.fromStackTraceToString(e.getStackTrace()));
		} catch (ParserConfigurationException e) {
			throw new AssertionFailedError(BasicIOTestCase.fromStackTraceToString(e.getStackTrace()));
		} catch (IOException e) {
			throw new AssertionFailedError(BasicIOTestCase.fromStackTraceToString(e.getStackTrace()));
		}
		
		SimpleKnot sk2 = new SimpleKnot(diagram2);
		Polynom kaufman2 = sk2.getKaufman();
		Polynom jonesKaufman2 = sk2.getJonesKaufman();
		TestCase.assertEquals("test12_4",
				"1 * x^-17 - 1 * x^-13 + 3 * x^-9 - 4 * x^-5 + 5 * x^-1 - 5 * x^3 + 5 * x^7 - 3 * x^11 + 2 * x^15 - 1 * x^19",
				kaufman2.toString());
		TestCase.assertEquals("test12_5",
				"- 1 * x^-14 + 1 * x^-10 - 3 * x^-6 + 4 * x^-2 - 5 * x^2 + 5 * x^6 - 5 * x^10 + 3 * x^14 - 2 * x^18 + 1 * x^22",
				jonesKaufman2.toString());
		TestCase.assertEquals("test12_6",
				"- x<sup>7/2</sup> + x<sup>5/2</sup> - 3x<sup>3/2</sup> + 4x<sup>1/2</sup> - 5x<sup>-1/2</sup> + 5x<sup>-3/2</sup> - 5x<sup>-5/2</sup> + 3x<sup>-7/2</sup> - 2x<sup>-9/2</sup> + x<sup>-11/2</sup>",
				sk2.getJonesInHTML());
		
		TestCase.assertEquals("test12_7", kaufman1.toString(), kaufman2.toString());
		TestCase.assertEquals("test12_8", jonesKaufman2.toString(), jonesKaufman2.toString());
		TestCase.assertEquals("test12_9", sk1.getJonesInHTML(), sk2.getJonesInHTML());
	}
}
